#include <sys/types.h>
#include <sys/stat.h>

#include <errno.h>
#include <fcntl.h>
#include <stdlib.h>
#include <stdint.h>
#include <unistd.h>

#include "mu.h"

/* Return 0 on success, -1 on error */
int
cat_file(const char *path)
{
    int fd = -1;
    int ret = 0;
    ssize_t nr = 0, nw = 0;
    uint8_t buf[64];

    fd = open(path, O_RDONLY);
    if (fd == -1) {
        ret = -1;
        mu_stderr_errno(errno, "%s", path);
        goto out;
    }

    while ((nr = read(fd, buf, sizeof(buf))) > 0) {
        nw = write(STDOUT_FILENO, buf, nr);;
        if (nw == -1) {
            ret = -1;
            mu_stderr_errno(errno, "error writing to stdout");
            goto out;
        }

        if (nw != nr) {
            ret = -1;
            mu_stderr("partial write to stdout");
            goto out;
        }
    }

    if (nr == -1) {
        ret = -1;
        mu_stderr_errno(errno, "error reading %s", path);
    }

out:
    if (fd != -1)
        close(fd);

    return ret;
}


int
main(int argc, char *argv[])
{
    int ret = EXIT_SUCCESS;
    int i;
    int err;

    if (argc < 2)
        mu_die("Usage: %s PATHS...", argv[0]);

    for (i = 1; i < argc; i++) {
        err = cat_file(argv[i]);
        if (err != 0) {
            ret = EXIT_FAILURE;
        }
    }
        
    return ret;
}
